package api

// Set is a simple unordered collection of strings.
type Set map[string]struct{}

// Add a value to the set.
func (s Set) Add(value string) {
	s[value] = struct{}{}
}

// Discard a value from the set.
func (s Set) Discard(value string) {
	if _, ok := s[value]; ok {
		delete(s, value)
	}
}

// Intersect returns a new Set that is the intersection of s and the
// specified argument.
func (s Set) Intersect(other Set) Set {
	var a, b Set
	if len(s) < len(other) {
		a = s
		b = other
	} else {
		a = other
		b = s
	}

	out := make(Set)
	for value, _ := range a {
		if _, ok := b[value]; ok {
			out[value] = struct{}{}
		}
	}

	return out
}

// Update merges a Set into this one.
func (s Set) Update(other Set) {
	for value, _ := range other {
		s.Add(value)
	}
}
