package upload

import (
	"errors"

	"git.autistici.org/ale/djrandom/util/metadata_extraction"
)

// GetFingerprint is a fingerprint function for audio files, which
// uses the 'metadata_extraction' package.
func GetFingerprint(fileinfo *FileInfo) (string, error) {
	// If we want this to work, some previous stage in the filtering
	// pipeline had better set the 'mime-type' key in the Meta map.
	mimeType, ok := fileinfo.Meta["mime-type"]
	if !ok {
		return "", errors.New("No mime-type defined")
	}

	af, _, err := metadata_extraction.GetMetadata(fileinfo.Path, mimeType)
	if err != nil {
		return "", err
	}

	// For some reason, in certain cases GetMetadata() returns
	// successfully with an invalid fingerprint (all zeros).
	if af.MD5 == "00000000000000000000000000000000" {
		return "", errors.New("Invalid fingerprint (all zeros)")
	}

	return af.MD5, nil
}
