// Background processing daemon for djrandom.

package main

import (
	"flag"
	"log"

	"git.autistici.org/ale/djrandom/server/actions"
	"git.autistici.org/ale/djrandom/services/database"
	"git.autistici.org/ale/djrandom/services/index"
	"git.autistici.org/ale/djrandom/services/storage"
	"git.autistici.org/ale/djrandom/services/tasks"
	"git.autistici.org/ale/djrandom/util"
	"git.autistici.org/ale/djrandom/util/daemon"
)

var (
	config     = util.NewConfig()
	configFile = flag.String("config", "/etc/djrandom/server.conf", "Config file location")
	addr       = flag.String("addr", ":3006", "TCP address to listen on")
	nWorkers   = flag.Int("nworkers", 3, "Number of workers")
)

func main() {
	flag.Parse()

	if err := config.Read(*configFile); err != nil {
		log.Fatalf("Error: could not read config file %s: %s", *configFile, err.Error())
	}

	daemon.Setup()

	// Create client stubs for all our services.
	dbImpl := database.NewDatabaseImplFromConfig(config)
	storageClient := storage.NewDistributedStorageClientFromConfig(config)
	indexClient := index.NewIndexClientFromConfig(config)
	taskClient := tasks.NewTaskClientFromConfig(config, dbImpl)

	// Start the state machine.
	sm := actions.NewSongStateMachine(dbImpl, storageClient, indexClient, taskClient)
	go sm.Run(*nWorkers)

	// Start an HTTP server (just for the debug pages, really).
	log.Printf("Starting processing server on %s", *addr)
	err := util.ListenAndServe(*addr, nil)
	if err != nil {
		log.Fatal("ListenAndServe: ", err)
	}
}
