package frontend

import (
	"encoding/json"
	"log"
	"net/http"

	"git.autistici.org/ale/djrandom/api"
)

const (
	globalMarkovUser = "@GLOBAL"
)

func findNextSong(curSong string, r *djRequest) string {
	// Retrieve the Markov map for this user first.
	for _, user := range []string{r.AuthUser, globalMarkovUser} {
		mmap, ok := r.Ctx.Db.GetMarkovMap(nil, user)
		if !ok {
			continue
		}

		song, ok := mmap.GetSong(curSong)
		if ok {
			return song
		}
	}
	return ""
}

func Suggest(w http.ResponseWriter, r *djRequest) {
	var suggestReq api.SuggestRequest
	if err := json.NewDecoder(r.Request.Body).Decode(&suggestReq); err != nil {
		log.Printf("Suggest(): Bad request: %s", err)
		http.Error(w, "Bad Request", http.StatusBadRequest)
		return
	}

	resp := api.SuggestResponse{
		SongIds: make([]string, 0),
	}

	if len(suggestReq.CurrentSongs) > 0 {
		curSong := suggestReq.CurrentSongs[len(suggestReq.CurrentSongs)-1]
		nextSong := findNextSong(curSong, r)
		if nextSong != "" {
			resp.SongIds = append(resp.SongIds, nextSong)
		}
	}

	sendJsonResponse(w, &resp)
}
