// Cleanup completed jobs.
//
package main

import (
	"flag"
	"log"
	"time"

	"github.com/dgryski/dmrgo"

	"git.autistici.org/ale/djrandom/services"
	"git.autistici.org/ale/djrandom/services/database"
	"git.autistici.org/ale/djrandom/util"

	"git.autistici.org/ale/djrandom/api/task_api"
)

var (
	config        = util.NewConfig()
	configFile    = flag.String("config", "/etc/djrandom/server.conf", "Config file location")
	cleanAllTasks = flag.Bool("all", false, "Clear all tasks (dangerous)")
	maxJobAge     = flag.Int64("max-age", 86400, "Task age cutoff")
)

type MRCleanTasks struct {
	db    services.Database
	proto dmrgo.StreamProtocol
	now   int64
}

func (mr *MRCleanTasks) Map(key string, value string, emitter dmrgo.Emitter) {
	jobId := value

	// Create a database session.
	session, err := mr.db.NewSession()
	if err != nil {
		return
	}
	defer session.Close()

	// Fetch the job and delete it if it's too old.
	if !*cleanAllTasks {
		var job task_api.Job
		if err := mr.db.Get(session, "tasks", jobId, &job); err != nil {
			return
		}
		if !job.Final || (mr.now-job.CompletedAt) < *maxJobAge {
			return
		}
	}

	mr.db.Del(session, "tasks", jobId)
	log.Printf("removed job %s", jobId)
}

func (mr *MRCleanTasks) MapFinal(emitter dmrgo.Emitter) {
}

func (mr *MRCleanTasks) Reduce(key string, values []string, emitter dmrgo.Emitter) {
}

func main() {
	flag.Parse()

	if err := config.Read(*configFile); err != nil {
		log.Fatalf("Error: could not read config file %s: %s", *configFile, err.Error())
	}

	dbImpl := database.NewDatabaseImplFromConfig(config)
	mr := &MRCleanTasks{
		db:    dbImpl,
		proto: &dmrgo.JSONProtocol{},
		now:   time.Now().Unix(),
	}
	dmrgo.Main(mr)
}
