package main

import (
	"flag"
	"fmt"
	"log"

	"git.autistici.org/ale/djrandom/services/database"
	"git.autistici.org/ale/djrandom/util"
)

var (
	config     = util.NewConfig()
	configFile = flag.String("config", "/etc/djrandom/server.conf", "Config file location")
	bucket     = flag.String("bucket", "", "Bucket to dump")
)

func main() {
	flag.Parse()

	if err := config.Read(*configFile); err != nil {
		log.Fatalf("Error: could not read config file %s: %s", *configFile, err.Error())
	}

	if *bucket == "" {
		log.Fatal("Must specify --bucket!")
	}

	dbClient := database.NewDatabaseRpcClient(config.MustGetString("db_addr"))
	session, err := dbClient.NewSession()
	if err != nil {
		log.Fatalf("Could not create db session: %s", err)
	}

	iter, err := session.ScanKeys(*bucket, "", "z", -1)
	if err != nil {
		log.Fatalf("Scan error: %s", err)
	}

	for {
		key, err := iter.Next(nil)
		if err != nil {
			if err != database.EOF {
				log.Fatalf("Error: %s", err)
			}
			break
		}
		fmt.Printf("%s\n", key)
	}
}
