package main

import (
	"flag"
	"log"

	"github.com/dgryski/dmrgo"

	"git.autistici.org/ale/djrandom/services"
	"git.autistici.org/ale/djrandom/services/database"
	"git.autistici.org/ale/djrandom/services/tasks"
	"git.autistici.org/ale/djrandom/util"

	"git.autistici.org/ale/djrandom/api/task_api"
	"git.autistici.org/ale/djrandom/server/actions/plans"
)

var (
	config     = util.NewConfig()
	configFile = flag.String("config", "/etc/djrandom/server.conf", "Config file location")
	planName   = flag.String("plan", "incoming", "Plan to execute")
)

type MRReprocess struct {
	db    services.Database
	tc    services.TaskClient
	plan  *task_api.Plan
	proto dmrgo.StreamProtocol
}

func (mr *MRReprocess) Map(key string, value string, emitter dmrgo.Emitter) {
	songId := value
	mr.tc.CreateJob(mr.plan, songId)
}

func (mr *MRReprocess) MapFinal(emitter dmrgo.Emitter) {
}

func (mr *MRReprocess) Reduce(key string, values []string, emitter dmrgo.Emitter) {
}

func main() {
	flag.Parse()

	if err := config.Read(*configFile); err != nil {
		log.Fatalf("Error: could not read config file %s: %s", *configFile, err.Error())
	}

	plan, ok := plans.KnownPlans[*planName]
	if !ok {
		log.Fatal("Unknown plan")
	}

	dbImpl := database.NewDatabaseImplFromConfig(config)
	taskClient := tasks.NewTaskClientFromConfig(config, dbImpl)
	mr := &MRReprocess{
		db:    dbImpl,
		tc:    taskClient,
		plan:  plan,
		proto: &dmrgo.JSONProtocol{},
	}
	dmrgo.Main(mr)
}
