package main

import (
	"flag"
	"log"

	"git.autistici.org/ale/djrandom/services/database"
	"git.autistici.org/ale/djrandom/services/database/backend_leveldb"
	"git.autistici.org/ale/djrandom/util"
	"git.autistici.org/ale/djrandom/util/daemon"
)

var (
	config     = util.NewConfig()
	configFile = flag.String("config", "/etc/djrandom/server.conf", "Config file location")
	addr       = flag.String("addr", ":3007", "TCP address to listen on.")
)

func main() {
	flag.Parse()

	if err := config.Read(*configFile); err != nil {
		log.Fatalf("Error: could not read config file %s: %s", *configFile, err.Error())
	}

	daemon.Setup()

	backend := backend_leveldb.NewLevelDbBackend(config.MustGetString("db_path"))
	dbsvc := &database.DatabaseService{Backend: backend}
	database.RegisterService(dbsvc)

	log.Printf("Starting database service on %s", *addr)
	err := util.ListenAndServe(*addr, nil)
	if err != nil {
		log.Fatal("ListenAndServe: ", err)
	}
}
