package main

import (
	"flag"
	"log"
	"time"

	"git.autistici.org/ale/djrandom/partition"
	"git.autistici.org/ale/djrandom/services/index"
	"git.autistici.org/ale/djrandom/util"
	"git.autistici.org/ale/djrandom/util/daemon"
)

var (
	config     = util.NewConfig()
	configFile = flag.String("config", "/etc/djrandom/server.conf", "Config file location")
	addr       = flag.String("addr", ":3002", "TCP address to listen on.")
)

func main() {
	flag.Parse()

	if err := config.Read(*configFile); err != nil {
		log.Fatalf("Error: could not read config file %s: %s", *configFile, err.Error())
	}

	daemon.Setup()

	indexPmap := partition.MustLoadPartitionMap(
		config.MustGetString("index_pmap"))
	indexPmap.HandleDebug("index")

	idxsvc := index.NewIndexService(
		config.MustGetString("index_file"),
		indexPmap,
		config.MustGetString("index_self"),
	)

	// Start rebalance worker and stats collector.
	partition.RegisterStatsService(idxsvc)
	go partition.RunRebalancer(idxsvc, time.Second*900)

	index.RegisterService(idxsvc)

	log.Printf("Starting index service on %s", *addr)
	err := util.ListenAndServe(*addr, nil)
	if err != nil {
		log.Fatal("ListenAndServe: ", err)
	}
}
