package main

import (
	"flag"
	"log"
	"time"

	"git.autistici.org/ale/djrandom/services/database"
	"git.autistici.org/ale/djrandom/services/tasks"
	"git.autistici.org/ale/djrandom/util"
	"git.autistici.org/ale/djrandom/util/daemon"
)

var (
	config     = util.NewConfig()
	configFile = flag.String("config", "/etc/djrandom/server.conf", "Config file location")
	addr       = flag.String("addr", ":3005", "TCP address to listen on.")
)

func main() {
	flag.Parse()

	if err := config.Read(*configFile); err != nil {
		log.Fatalf("Error: could not read config file %s: %s", *configFile, err.Error())
	}

	daemon.Setup()

	db := database.NewDatabaseImplFromConfig(config)
	ts := tasks.NewTaskServer(db)
	ts.RegisterService()

	log.Printf("Starting task queue service on %s", *addr)
	err := util.ListenAndServeWithTimeout(*addr, nil, 1800*time.Second)
	if err != nil {
		log.Fatal("ListenAndServe: ", err)
	}
}
