// Player demo with buzz.js

var pl = {};

djrandom.EventObject = function() {
  var events = {};

  this.emit = function(event) {
    if (!events[event]) return;

    var args = Array.prototype.slice.call(arguments, 1),
    callbacks = events[event];

    for (var i = 0, len = callbacks.length; i < len; i++) {
      callbacks[i].apply(null, args);
    }
  };

  this.on = function(event, fn) {
    events[event] || (events[event] = []);
    events[event].push(fn);
  };

  this.off = function(event, fn) {
    var eventsOf = events[event],
    index = eventsOf.indexOf(fn);
      
    ~index && eventsOf.splice(index, 1);
  };
};


djrandom.PlayerUI = function() {

  var state = 'paused',
      el = $('.player .button'),
      ev = new djrandom.EventObject();

  function updateui() {
    if (state == 'playing') {
      el.toggleClass('pause', true);
    } else {
      el.toggleClass('pause', false);
    }
  }

  function toggleState() {
    if (state == 'playing') {
      state = 'paused';
      ev.emit('pause');
    } else {
      state = 'playing';
      ev.emit('play');
    }
    updateui();
  }

  el.click(function() {
    toggleState();
  });

  this.SetState = function(playing) {
    if (playing) {
      state = 'playing';
    } else {
      state = 'paused';
    }
    updateui();
  };

  this.GetState = function() {
    return state;
  };

  this.on = function(event, fn) {
    ev.on(event, fn);
  };
};

djrandom.Player = function() {

  var sound = null,
      playing = false,
      ev = new djrandom.EventObject();

  // Play a new song.
  this.Play = function(song) {
    console.log('playing(' + song.id + ')...');
    var url = '/dl/' + song.audio_file.md5;
    newSound = new buzz.sound(url, {preload: true});
    ev.emit('loading', song);

    // 'next' is raised when a song ends.
    newSound.bind('ended', function(e) {
      ev.emit('next');
    });
    // 'error' is raised on errors.
    newSound.bind('error', function(e) {
      console.log('Error: ' + this.getErrorMessage());
      ev.emit('pause');
      ev.emit('error');
      this.stop();
    });
    // 'pause' is raised when no audio is being output. It is a UI signal.
    newSound.bind('ended pause', function(e) {
      // emit the pause signal to the UI
      console.log('ended/pause');
      ev.emit('pause');
    });
    // 'play' is raised when audio is playing. It is a UI signal.
    newSound.bind('playing', function(e) {
      // emit the play signal to the UI
      console.log('play');
      ev.emit('play');
    });

    // Play the new sound, possibly with a faded transition.
    ev.emit('playing', song);
    if (sound && !sound.isEnded() && !sound.isPaused()) {
      sound.unbind('ended pause');
      sound.fadeWith(newSound.play(), 200);
    } else {
      newSound.play().fadeIn(200);
    }

    sound = newSound;
  };

  // Pauses *and* unpauses the currently playing sound.
  this.Pause = function() {
    if (sound) {
      sound.togglePlay();
    }
  };

  // Stop the current sound and destroy it.
  this.Stop = function() {
    if (sound) {
      console.log('stopping...');
      sound.stop();
      sound = null;
    }
  };

  this.on = function(event, fn) {
    ev.on(event, fn);
  };

};


djrandom.Playlist = function(api, playlistId) {

  var songTemplate = _.template($('#tpl-song').html()),
    playlistDiv = $('#' + playlistId),
    songList = [],
    songHistory = [],
    maxSongHistoryLen = 3,
    curSong = null,
    dedupMeta = true,
    ev = new djrandom.EventObject();

  function isInSongList(song) {
    for (var i = 0; i < songList.length; i++) {
      if (songList[i].id == song.id || (dedupMeta && songList[i].meta.artist == song.meta.artist && songList[i].meta.title == song.meta.title)) {
        return true;
      }
    }
    return false;
  }

  function getSongAfter(songId) {
    for (var i = 0; i < songList.length - 1; i++) {
      if (songList[i].id == songId) {
        return songList[i + 1];
      }
    }
    return null;
  }

  function deleteFromSongList(songId) {
    console.log('Playlist: deleting song ' + songId);
    var newSongList = [];
    $.each(songList, function(idx, song) {
      if (song.id != songId) {
        newSongList.push(song);
      }
    });
    songList = newSongList;
    $('#song_' + songId).remove();
  }

  function addToSongHistory(songId) {
    songHistory.push(songId);
    if (songHistory.length > maxSongHistoryLen) {
      songHistory.splice(0, 1);
    }
  }

  function doPlay(song) {
    ev.emit('play', song);
    curSong = song.id;
    $('.song').toggleClass('playing', false);
    $('#song_' + song.id).toggleClass('playing', true);
  }

  function renderSong(song) {
    var songHtml = $(songTemplate({song: song}));
    songHtml.find('.title').click(function() {
      console.log('Playlist: play ' + song.id);
      doPlay(song);
    });
    songHtml.find('.song-remove').click(function() {
      deleteFromSongList(song.id);
    });
    return songHtml;
  }

  // Playlist methods. 
  this.AddSong = function(song) {
    if (isInSongList(song)) {
      console.log('Playlist: song ' + song.id + ' already present');
      return;
    }
    // Verify that the incoming 'song' object has the 'audio_file'
    // attribute set, so that we know which file to play.
    if (!song.audio_file) {
      console.log('song ' + song.id + ' has no good audio file');
      return;
    }
    console.log('Playlist: adding song ' + song.id);
    songList.push(song);
    var songHtml = renderSong(song);
    songHtml.appendTo(playlistDiv);
  };

  this.DeleteSong = function(songId) {
    deleteFromSongList(songId);
  };

  this.Clear = function() {
    songList = [];
    playlist.html('');
  };

  this.Play = function(song) {
    doPlay(song);
  };

  this.PlayNextSong = function() {
    // Current song has played succesfully, record this fact.
    addToSongHistory(curSong);
    api.AddToPlayLog(songHistory);

    // Play the next song, if any.
    var newSong = getSongAfter(curSong);
    if (newSong) {
      console.log('Next song: ' + newSong.id);
      doPlay(newSong);
    } else {
      console.log('No more songs...');
    }
  };

  this.GetFacets = function() {
    var facets = {};
    $.each(songList, function(idx, song) {
      if (song.meta.artist) {
        var artist = song.meta.artist;
        if (!facets[artist]) {
          facets[artist] = {count: 1, albums: {}};
        } else {
          facets[artist].count++;
        }

        if (song.meta.album) {
          var album = song.meta.album;
          if (!facets[artist].albums[album]) {
            facets[artist].albums[album] = {count: 1};
          } else {
            facets[artist].albums[album].count++;
          }
        }
      }
    });
    return facets;
  };

  this.on = function(event, fn) {
    ev.on(event, fn);
  };
};
