package util

import (
	"encoding/json"
	"io/ioutil"
	"log"
	"os"
)

type Config struct {
	values map[string]string
}

func NewConfig() *Config {
	return &Config{make(map[string]string)}
}

func (c *Config) Read(path string) error {
	file, err := os.Open(path)
	if err != nil {
		return err
	}
	defer file.Close()
	return json.NewDecoder(file).Decode(&(c.values))
}

func (c *Config) replaceWithFile(value string) string {
	if len(value) > 1 && value[0] == '<' {
		path := value[1:]
		data, err := ioutil.ReadFile(path)
		if err != nil {
			log.Fatalf("Fatal error: could not read data from '%s': %s", path, err)
		}
		return string(data)
	}
	return os.ExpandEnv(value)
}

func (c *Config) GetString(key string, defaultValue string) string {
	if value, ok := c.values[key]; ok {
		return c.replaceWithFile(value)
	}
	return defaultValue
}

func (c *Config) MustGetString(key string) string {
	if value, ok := c.values[key]; ok {
		return c.replaceWithFile(value)
	}
	log.Fatalf("Fatal error: configuration is missing a value for '%s'", key)
	return "" // unreached
}
