package ffmpeg

import (
	"bytes"
	"flag"
	"io"
	"log"
	"os/exec"
)

var ffmpegBinary = flag.String("ffmpeg-binary", "avconv", "Ffmpeg/avconv binary")

// Run invokes ffmpeg with arguments and data on stdin.
// Returns output and exit status.
func Run(input io.Reader, args []string) (string, error) {
	var b bytes.Buffer
	cmd := exec.Command(*ffmpegBinary, args...)
	log.Printf("Exec: %v", cmd)

	cmd.Stdout = &b
	cmd.Stderr = &b
	stdin, err := cmd.StdinPipe()
	if err != nil {
		return "", err
	}

	if err = cmd.Start(); err != nil {
		stdin.Close()
		return "", err
	}
	io.Copy(stdin, input)
	stdin.Close()

	err = cmd.Wait()
	return b.String(), err
}
