package tokenizer

import (
	"regexp"
)

var (
	// Unicode-safe RE2 alphanumeric token match.
	wordRegexp = regexp.MustCompile(`[\pL\pN]+`)
)

// Tokenize splits a string into alphanumeric tokens.
func Tokenize(s string) []string {
	return wordRegexp.FindAllString(s, -1)
}

// TokenFilter applies a filter function to a list of tokens.
func TokenFilter(in []string, f func(string) bool) []string {
	out := []string{}
	for _, t := range in {
		if f(t) {
			out = append(out, t)
		}
	}
	return out
}

// IsKeyword returns True if the token looks useful.
func IsKeyword(t string) bool {
	return len(t) > 1
}
